from __future__ import annotations

from typing import Literal, NewType, Optional

from typing_extensions import NotRequired, Unpack

from haitch._attrs import Attributes, GlobalAttrs
from haitch._void_elements._void_element import VoidElement

LinkElement = NewType("LinkElement", VoidElement)
"""A `<link>` element."""


def link(
    *,
    extra_attrs: Optional[Attributes] = None,
    **attrs: Unpack[LinkAttrs],
) -> LinkElement:
    """Specifies relationships between current document and external resource.

    This element is most commonly used to link to stylesheets, but is also used
    to establish site icons (both "favicon" style icons and icons for the home
    screen and apps on mobile devices) among other things.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/link>
    """
    el = VoidElement("link")(**attrs, **extra_attrs or {})
    return LinkElement(el)


class LinkAttrs(GlobalAttrs):
    """Attributes for the `<link>` element.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/link#attributes>
    """

    href: str
    """Specifies the URL of the linked resource.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/link#href>
    """

    rel: str
    """Relationship between a linked resource and the current document.

    Valid values: <https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/rel>

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/link#rel>
    """

    as_: NotRequired[str]
    """Specifies the type of content being loaded.

    This attribute is required when `rel="preload"` has been set on the `<link>`
    element, optional when `rel="modulepreload"` has been set, and otherwise
    should not be used.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/link#as>
    """

    crossorigin: NotRequired[Literal["anonymous", "use-credentials"]]
    """Indicates whether CORS must be used when fetching the resource.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/link#crossorigin>
    """

    fetchpriority: NotRequired[Literal["high", "low", "auto"]]
    """Provides a hint of the relative priority to use when fetching a preloaded resource.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/link#fetchpriority>
    """

    hreflang: NotRequired[str]
    """Indicates the language of the linked resource.

    Note: use only if `href` attribute is present.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/link#hreflang>
    """

    integrity: NotRequired[str]
    """Inline-metadata to verify that resource has not been manipulated.

    This can be a base64-encoded cryptographic hash of the resource (file) to be
    fetched.

    Note: An integrity value may contain multiple hashes separated by
    whitespace. A resource will be loaded if it matches one of those hashes.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/link#integrity>
    """

    media: NotRequired[str]
    """Specifies the media that the linked resource applies to.

    Its value must be a media type / media query. This attribute is mainly
    useful when linking to external stylesheets — it allows the user agent to
    pick the best adapted one for the device it runs on.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/link#media>
    """

    referrerpolicy: NotRequired[
        Literal[
            "no-referrer",
            "no-referrer-when-downgrade",
            "origin",
            "origin-when-cross-origin",
            "unsafe-url",
        ]
    ]
    """Indicates which referrer to use when fetching the resource.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/link#referrerpolicy>
    """

    sizes: NotRequired[str]
    """Sizes of the icons for visual media contained in the resource.

    It must be present only if the `rel` contains a value of `icon` or a
    non-standard type such as Apple's `apple-touch-icon`. It may have the
    following values:

        - "any": can be scaled to any size, i.e. `image/svg+xml` format.
        - "<width>x<height>": each size is in pixels, i.e. "200x200".

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/link#sizes>
    """

    type_: NotRequired[str]
    """Defines the type of the content linked to.

     The value of the attribute should be a MIME type such as text/html,
     text/css, and so on. The common use of this attribute is to define the type
     of stylesheet being referenced (such as text/css), but given that CSS is
     the only stylesheet language used on the web, not only is it possible to
     omit the type attribute, but is actually now recommended practice. It is
     also used on rel="preload" link types, to make sure the browser only
     downloads file types that it supports.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/link#type>
    """



from __future__ import annotations

from typing import NewType, Optional

from typing_extensions import Unpack

from haitch._attrs import Attributes, GlobalAttrs
from haitch._elements._element import Element
from haitch._typing import Child

MarkElement = NewType("MarkElement", Element)
"""A `<mark>` element."""


def mark(
    *children: Child,
    extra_attrs: Optional[Attributes] = None,
    **attrs: Unpack[GlobalAttrs],
) -> MarkElement:
    """Represents text which highlighted for reference/notation purposes.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/mark>
    """
    el = Element("mark")(**attrs, **extra_attrs or {})(*children)
    return MarkElement(el)



from __future__ import annotations

from typing import NewType, Optional

from typing_extensions import Unpack

from haitch._attrs import Attributes, GlobalAttrs
from haitch._elements._element import Element
from haitch._typing import Child

PreElement = NewType("PreElement", Element)
"""A `<pre>` element."""


def pre(
    *children: Child,
    extra_attrs: Optional[Attributes] = None,
    **attrs: Unpack[GlobalAttrs],
) -> PreElement:
    """Represents preformatted text presented exactly as written.

    The text is typically rendered using a non-proportional, or monospaced,
    font. Whitespace inside this element is displayed as written.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/pre>
    """
    el = Element("pre")(**attrs, **extra_attrs or {})(*children)
    return PreElement(el)



from __future__ import annotations

from typing import Literal, NewType, Optional

from typing_extensions import NotRequired, Unpack

from haitch._attrs import Attributes, GlobalAttrs
from haitch._elements._element import Element
from haitch._typing import Child

ThElement = NewType("ThElement", Element)
"""A `<th>` element."""


def th(
    *children: Child,
    extra_attrs: Optional[Attributes] = None,
    **attrs: Unpack[ThAttrs],
) -> ThElement:
    """Defines a cell as the header of a group of table cells.

    This group is defined by the `scope` and `headers` attribute.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/th>
    """
    el = Element("th")(**attrs, **extra_attrs or {})(*children)
    return ThElement(el)


class ThAttrs(GlobalAttrs):
    """Attributes for the `<th>` element.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/th#attributes>
    """

    abbr: NotRequired[str]
    """A short, abbreviated description of the header cell's content.

    It is provided as an alternative label to use for the header cell when
    referencing the cell in other contexts. Some user-agents, such as speech
    readers, may present this description before the content itself.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/th#abbr>
    """

    colspan: NotRequired[int]
    """A non-negative integer value indicating the number of columns.

    If not provided, the default value is `1`.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/th#colspan>
    """

    headers: NotRequired[str]
    """Provides the headers for this header cell.

    A list of space-separated strings corresponding to the `id` attributes of
    the `<th>` elements.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/th#headers>
    """

    rowspan: NotRequired[int]
    """A non-negative integer value indicating the number of rows.

    If not provided, the default value is `1`. if its value is set to 0, the
    header cell will extends to the end of the table grouping section (<thead>,
    <tbody>, <tfoot>, even if implicitly defined), that the <th> belongs to.
    Values higher than 65534 are clipped at 65534.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/th#rowspan>
    """

    scope: NotRequired[Literal["row", "col", "rowgroup", "colgroup"]]
    """Defines the cells that the header element relates to.

    If the `scope` attribute is not specified, or its value is not `row`, `col`,
    `rowgroup`, or `colgroup`, then browsers automatically select the set of
    cells to which the header cell applies.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/th#scope>
    """



from __future__ import annotations

from typing import Literal, NewType, Optional

from typing_extensions import NotRequired, Unpack

from haitch._attrs import Attributes, GlobalAttrs
from haitch._elements._element import Element
from haitch._typing import Child

OlElement = NewType("OlElement", Element)
"""A `<ol>` element."""


def ol(
    *children: Child,
    extra_attrs: Optional[Attributes] = None,
    **attrs: Unpack[OlAttrs],
) -> OlElement:
    """Represents an ordered list of items.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/ol>
    """
    el = Element("ol")(**attrs, **extra_attrs or {})(*children)
    return OlElement(el)


class OlAttrs(GlobalAttrs):
    """Attributes for the `<ol>` element.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/ol#attributes>
    """

    reversed: NotRequired[bool]
    """Specifies that the list's items are in reverse order.

    Items will be numbered from high to low.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/ol#reversed>
    """

    start: NotRequired[int]
    """An integer to start counting from for the list items.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/ol#reversed>
    """

    type_: NotRequired[Literal["a", "A", "i", "I", "1"]]
    """Sets the numbering type.

    Available numbering types:
        - `a` for lowercase letters
        - `A` for uppercase letters
        - `i` for lowercase Roman numerals
        - `I` for uppercase Roman numerals
        - `1` for numbers (default)

    Note: Unless the type of the list number matters (like legal or technical documents
    where items are referenced by their number/letter), use the CSS `list-style-type`
    property instead.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/ol#type>
    """



from pytest_snapshot.plugin import Snapshot

import haitch as H
from tests.helpers import get_snapshot_name, prettify


def test_form(snapshot: Snapshot) -> None:
    dom = H.form(action="", method="get")(
        H.fieldset(
            H.legend("Find users"),
            H.label(for_="name")("Enter name: "),
            H.input(type_="text", name="name", id_="name", required=True),
            H.button("Search"),
        ),
    )

    snapshot.assert_match(prettify(dom), get_snapshot_name())



from __future__ import annotations

from typing import NewType, Optional

from typing_extensions import NotRequired, Unpack

from haitch._attrs import Attributes, GlobalAttrs
from haitch._void_elements._void_element import VoidElement

SourceElement = NewType("SourceElement", VoidElement)
"""A `<source>` element."""


def source(
    *,
    extra_attrs: Optional[Attributes] = None,
    **attrs: Unpack[SourceAttrs],
) -> SourceElement:
    """Specifies one or more media resources for the media elements.

    Media elements: `<picture>`, `<audio>`, and `<video>`

    This element is commonly used to offer the same media content in multiple
    file formats in order to provide compatibility with a broad range of
    browsers given their differing support for image file formats and media file
    formats.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/source>
    """
    el = VoidElement("source")(**attrs, **extra_attrs or {})
    return SourceElement(el)


class SourceAttrs(GlobalAttrs):
    """Attributes for the `<source>` element.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/source#attributes>
    """

    type_: NotRequired[str]
    """Specifies the MIME media type of the image or other media type.

    Image types: <https://developer.mozilla.org/en-US/docs/Web/Media/Formats/Image_types>

    Other types: <https://developer.mozilla.org/en-US/docs/Web/Media/Formats/Containers>

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/source#type>
    """

    src: NotRequired[str]
    """Specifies the URL of the media resource.

    Note: required for `<audio>` or `<video>`, but not allowed if parent is
    `<picture>`.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/source#src>
    """

    srcset: NotRequired[str]
    """Comma-separated list of one or more image URLs and their descriptors.

    Note: required if the parent of `<source>` is `<picture>`, but not allowed
    if parent is `<audio>` or `<video>`.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/source#srcset>
    """

    sizes: NotRequired[str]
    """List of source sizes describing the final rendered width of the image.

    Note: allowed if parent of `<source>` is `<picture>`, but not allowed if parent
    is `<audio>` or `<video>`.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/source#sizes>
    """

    media: NotRequired[str]
    """Specifies the media query for the resource's intended media.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/source#media>
    """

    height: NotRequired[int]
    """The displayed height of the resource, in CSS pixels.

    Note: allowed if parent of `<source>` is `<picture>`, but not allowed if
    parent is `<audio>` or `<video>`.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/source#height>
    """

    width: NotRequired[int]
    """The displayed width of the resource, in CSS pixels.

    Note: allowed if parent of `<source>` is `<picture>`, but not allowed if
    parent is `<audio>` or `<video>`.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/source#width>
    """



from pytest_snapshot.plugin import Snapshot

import haitch as H
from tests.helpers import get_snapshot_name, prettify


def test_multimedia(snapshot: Snapshot) -> None:
    dom = H.fragment(
        H.video(
            H.source(src="friday.webm", type_="video/webm"),
            H.track(default=True, src="friday.vtt"),
        ),
        H.figure(
            H.img(src="elephant.jpg", alt="Elephant at sunset"),
            H.figcaption("An elephant at sunset"),
        ),
    )

    snapshot.assert_match(prettify(dom), get_snapshot_name())



from __future__ import annotations

from typing import NewType, Optional

from typing_extensions import Unpack

from haitch._attrs import Attributes, GlobalAttrs
from haitch._elements._element import Element
from haitch._typing import Child

TitleElement = NewType("TitleElement", Element)
"""A `<title>` element."""


def title(
    *children: Child,
    extra_attrs: Optional[Attributes] = None,
    **attrs: Unpack[GlobalAttrs],
) -> TitleElement:
    """Defines the document's title.

    The content of the title is shown in a browser's title bar or page's tab.

    Note: it only contains text; tags within the element are ignored.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/title>
    """
    el = Element("title")(**attrs, **extra_attrs or {})(*children)
    return TitleElement(el)



"""
haitch - simplify your HTML building.
=====================================

Import haitch like so:

>>> import haitch as H

Now you have access to any element you like:

>>> H.img(src="my-image.png", alt="cool image") # valid
>>> H.foo("Custom element!") # valid

Both these statements would render correctly; however, the `<img>`
element would have typing and documentation, while the `<foo>` one
would not because it is not a standard element.

Lazily build a dom tree by passing children (args) and/or
attributes (kwargs):

>>> dom = H.a(
...     H.strong("Check out my website"), # arg
...     href="https://example.com", # kwarg
... )

A called element validates its input, attaches the values to itself,
and then returns itself. This enables us to chain calls together:

>>> dom = H.a(href="https://example.com")(
...     H.strong("Check out my website"),
... )

Both instances of `dom` will render the same HTML. I prefer the second
way because it looks more like HTML, and it keeps the attributes closer
to the parent element.

Until now we have only lazily built the dom tree. In order to render it
to HTML, you need to invoke the underlying `__str__` method:

>>> str(dom) # or print(dom)
<a href="https://example.com">
  <strong>Check out my website</strong>
</a>
"""

from haitch._components._html5 import html5
from haitch._elements._a import AnchorElement, a
from haitch._elements._article import ArticleElement, article
from haitch._elements._aside import AsideElement, aside
from haitch._elements._body import BodyElement, body
from haitch._elements._details import DetailsElement, details
from haitch._elements._div import DivElement, div
from haitch._elements._element import Element
from haitch._elements._figcaption import FigcaptionElement, figcaption
from haitch._elements._figure import FigureElement, figure
from haitch._elements._footer import FooterElement, footer
from haitch._elements._form import FormElement, form
from haitch._elements._h1 import H1Element, h1
from haitch._elements._h2 import H2Element, h2
from haitch._elements._h3 import H3Element, h3
from haitch._elements._h4 import H4Element, h4
from haitch._elements._h5 import H5Element, h5
from haitch._elements._h6 import H6Element, h6
from haitch._elements._head import HeadElement, head
from haitch._elements._header import HeaderElement, header
from haitch._elements._html import HtmlElement, html
from haitch._elements._label import LabelElement, label
from haitch._elements._li import LiElement, li
from haitch._elements._main import MainElement, main
from haitch._elements._mark import MarkElement, mark
from haitch._elements._nav import NavElement, nav
from haitch._elements._noscript import NoscriptElement, noscript
from haitch._elements._ol import OlElement, ol
from haitch._elements._p import ParagraphElement, p
from haitch._elements._pre import PreElement, pre
from haitch._elements._script import ScriptElement, script
from haitch._elements._section import SectionElement, section
from haitch._elements._span import SpanElement, span
from haitch._elements._style import StyleElement, style
from haitch._elements._summary import SummaryElement, summary
from haitch._elements._table import TableElement, table
from haitch._elements._td import TdElement, td
from haitch._elements._th import ThElement, th
from haitch._elements._time import TimeElement, time
from haitch._elements._title import TitleElement, title
from haitch._elements._tr import TrElement, tr
from haitch._elements._ul import UlElement, ul
from haitch._magic_elements._fragment import fragment
from haitch._magic_elements._unsafe import unsafe
from haitch._typing import Child, Html, SupportsHtml
from haitch._void_elements._area import AreaElement, area
from haitch._void_elements._base import BaseElement, base
from haitch._void_elements._br import BrElement, br
from haitch._void_elements._col import ColElement, col
from haitch._void_elements._embed import EmbedElement, embed
from haitch._void_elements._hr import HrElement, hr
from haitch._void_elements._img import ImgElement, img
from haitch._void_elements._input import InputElement, input
from haitch._void_elements._link import LinkElement, link
from haitch._void_elements._meta import MetaElement, meta
from haitch._void_elements._source import SourceElement, source
from haitch._void_elements._track import TrackElement, track
from haitch._void_elements._void_element import VoidElement
from haitch._void_elements._wbr import WbrElement, wbr

__all__ = [
    "AnchorElement",
    "AreaElement",
    "ArticleElement",
    "AsideElement",
    "BaseElement",
    "BodyElement",
    "BrElement",
    "Child",
    "ColElement",
    "DetailsElement",
    "DivElement",
    "Element",
    "EmbedElement",
    "FigcaptionElement",
    "FigureElement",
    "FooterElement",
    "FormElement",
    "FormElement",
    "H1Element",
    "H2Element",
    "H3Element",
    "H4Element",
    "H5Element",
    "H6Element",
    "HeadElement",
    "HeaderElement",
    "HrElement",
    "Html",
    "HtmlElement",
    "ImgElement",
    "InputElement",
    "LabelElement",
    "LiElement",
    "LinkElement",
    "MainElement",
    "MarkElement",
    "MetaElement",
    "NavElement",
    "NoscriptElement",
    "OlElement",
    "ParagraphElement",
    "PreElement",
    "ScriptElement",
    "SectionElement",
    "SourceElement",
    "SpanElement",
    "StyleElement",
    "SummaryElement",
    "SupportsHtml",
    "TableElement",
    "TdElement",
    "ThElement",
    "TimeElement",
    "TitleElement",
    "TrElement",
    "TrackElement",
    "UlElement",
    "VoidElement",
    "WbrElement",
    "a",
    "area",
    "article",
    "aside",
    "base",
    "body",
    "br",
    "col",
    "details",
    "div",
    "embed",
    "figcaption",
    "figure",
    "footer",
    "form",
    "form",
    "fragment",
    "h1",
    "h2",
    "h3",
    "h4",
    "h5",
    "h6",
    "head",
    "header",
    "hr",
    "html",
    "html5",
    "img",
    "input",
    "label",
    "li",
    "link",
    "main",
    "mark",
    "meta",
    "nav",
    "noscript",
    "ol",
    "p",
    "pre",
    "script",
    "section",
    "source",
    "span",
    "style",
    "summary",
    "table",
    "td",
    "th",
    "time",
    "title",
    "tr",
    "track",
    "ul",
    "unsafe",
    "wbr",
]


def __getattr__(tag: str) -> Element:
    return Element(tag)



from __future__ import annotations

from typing import Literal, NewType, Optional

from typing_extensions import NotRequired, Unpack

from haitch._attrs import Attributes, GlobalAttrs
from haitch._void_elements._void_element import VoidElement

TrackElement = NewType("TrackElement", VoidElement)
"""A `<track>` element."""


def track(
    *,
    extra_attrs: Optional[Attributes] = None,
    **attrs: Unpack[TrackAttrs],
) -> TrackElement:
    """Embeds timed text tracks.

    A common example is to automatically handle subtitles. The tracks are
    formatted in Web Video Text Tracks (WebVTT) format (`.vtt` files).

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/track>
    """
    el = VoidElement("track")(**attrs, **extra_attrs or {})
    return TrackElement(el)


class TrackAttrs(GlobalAttrs):
    """Attributes for the `<track>` element.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/track#attributes>
    """

    src: str
    """Address of the track (`.vtt` file).

    Note: must be a valid URL. This attribute must be specified and its URL
    value must have the same origin as the document - unless the `<audio>` or
    `<video>` parent element of the `track` element has a `crossorigin`
    attribute.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/track#src>
    """

    default: NotRequired[bool]
    """Indicates that the track should be enabled.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/track#default>
    """

    kind: NotRequired[
        Literal[
            "subtitles",
            "captions",
            "descriptions",
            "chapters",
            "metadata",
        ]
    ]
    """Indicates how the text track is meant to be used.

    If omitted, the default kind is `subtitles`. If the value is invalid,
    `metadata` will be used.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/track#kind>
    """

    label: NotRequired[str]
    """User-readable title used by browser when listing available text tracks.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/track#label>
    """

    srclang: NotRequired[str]
    """Language of the track text data.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/track#srclang>
    """



from __future__ import annotations

from typing import NewType, Optional

from typing_extensions import Unpack

from haitch._attrs import Attributes, GlobalAttrs
from haitch._elements._element import Element
from haitch._typing import Child

FigcaptionElement = NewType("FigcaptionElement", Element)
"""A `<figcaption>` element."""


def figcaption(
    *children: Child,
    extra_attrs: Optional[Attributes] = None,
    **attrs: Unpack[GlobalAttrs],
) -> FigcaptionElement:
    """Represents a caption/legend describing the contents of parent figure.

    Nesting this in a figure element improves accessibility.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/figcaption>
    """
    el = Element("figcaption")(**attrs, **extra_attrs or {})(*children)
    return FigcaptionElement(el)



import haitch as H


def test_simple_h1_example():
    # Render known `h1` tag.
    h1 = H.h1("Hello, world")

    got = str(h1)
    want = "<h1>Hello, world</h1>"

    assert got == want


def test_custom_component():
    # Render custom `foo` tag (useful for web components).
    foo = H.foo("Hello, world")

    got = str(foo)
    want = "<foo>Hello, world</foo>"

    assert got == want


def test_emails_example():
    # Fetch emails from data store
    emails = ["jane@aol.com", "bob@example.com", "mark@mail.org", "karen@hr.org"]

    # Build an unordered list of ".org" email addresses
    dom = H.ol(class_="email-list")(
        H.li(H.a(href=f"mailto:{email}")(email))
        for email in sorted(emails)
        if email.endswith(".org")
    )

    got = str(dom)
    want = '<ol class="email-list"><li><a href="mailto:karen@hr.org">karen@hr.org</a></li><li><a href="mailto:mark@mail.org">mark@mail.org</a></li></ol>'
    # Pipe this into prettier for improved readability: $ echo '...' | prettier --parser html

    assert got == want



from __future__ import annotations

from typing import NewType, Optional

from typing_extensions import NotRequired, Unpack

from haitch._attrs import Attributes, GlobalAttrs
from haitch._elements._element import Element
from haitch._typing import Child

TimeElement = NewType("TimeElement", Element)
"""A `<time>` element."""


def time(
    *children: Child,
    extra_attrs: Optional[Attributes] = None,
    **attrs: Unpack[TimeAttrs],
) -> TimeElement:
    """Represents a specific period in time.

    It may include the `datetime` attribute to translate dates into
    machine-readable format, allowing for better search engine results or custom
    feats such as reminders.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/time>
    """
    el = Element("time")(**attrs, **extra_attrs or {})(*children)
    return TimeElement(el)


class TimeAttrs(GlobalAttrs):
    """Attributes for the `<time>` element.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/time#attributes>
    """

    datetime: NotRequired[str]
    """Indicates he time and/or date of the element.

    The datetime must be in one of the formats described below:

        - `2011`
        - `2011-11`
        - `2011-11-18`
        - `11-18`
        - `2011-W47`
        - `14:54`
        - `14:54:39`
        - `14:54:39.929`
        - `2011-11-18T14:54:39.929`
        - `2011-11-18 14:54:39.929`
        - `2011-11-18T14:54:39.929Z`
        - `2011-11-18T14:54:39.929-0400`
        - `2011-11-18T14:54:39.929-04:00`
        - `2011-11-18 14:54:39.929Z`
        - `2011-11-18 14:54:39.929-0400`
        - `2011-11-18 14:54:39.929-04:00`

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/time#open>
    """



from __future__ import annotations

from typing import NewType, Optional

from typing_extensions import NotRequired, Unpack

from haitch._attrs import Attributes, GlobalAttrs
from haitch._void_elements._void_element import VoidElement

BaseElement = NewType("BaseElement", VoidElement)
"""A `<base>` element."""


def base(
    *,
    extra_attrs: Optional[Attributes] = None,
    **attrs: Unpack[BaseAttrs],
) -> BaseElement:
    """Specifies the base URL to use for all relative URLs in a document.

    Note: there can only be one `<base>` element in a document.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/base>
    """
    el = VoidElement("base")(**attrs, **extra_attrs or {})
    return BaseElement(el)


class BaseAttrs(GlobalAttrs):
    """Attributes for the `<base>` element.

    Warning: A `<base>` element must have an `href` attribute, a target
    attribute, or both. If at least one of these attributes are specified, the
    `<base>` element must come before other elements with attribute values that
    are URLs, such as a `<link>`'s `href` attribute.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/base#attributes>
    """

    href: NotRequired[str]
    """The base URL to be used throughout the document for relative URLs.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/base#href>
    """

    target: NotRequired[str]
    """A keyword or author-defined name of default browsing context to display.

    The following keywords have special meaning:

      - `_self` (default): load into same browsing content as current one.
      - `_blank`: load into a new unamed browsing context.
      - `_parent`: load into the parent browsing context.
      - `_top`: load into the top-level browsing context.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/base#target>
    """



from pytest_snapshot.plugin import Snapshot

import haitch as H
from tests.helpers import get_snapshot_name, prettify


def test_basic_document(snapshot: Snapshot) -> None:
    styles = """
    p {
        color: green;
    }
    """
    dom = H.html(lang="en")(
        H.head(
            H.meta(charset="utf-8"),
            H.meta(name="viewport", content="width=device-width, initial-scale=1"),
            H.meta(http_equiv="x-ua-compatible", content="ie=edge"),
            H.title("Basic page"),
            H.link(href="main.css", rel="stylesheet"),
            H.link(href="custom.css", rel="stylesheet"),
            H.script(src="main.js", defer=True),
            H.style(styles),
        ),
        H.body(class_="container")(
            H.noscript("JavaScript is not enabled. Good job."),
            H.div(
                H.p("This is a ", H.mark("mark"), "."),
                H.p("The date is ", H.time(datetime="2018-07-07")("July 7th")),
                H.a(href="#")("Hyperlink"),
                H.hr(),
                H.img(src="image.png", alt="cool image"),
                H.br(),
                H.ol(
                    H.li("First point"),
                    H.li("Second point"),
                ),
            ),
            H.pre("print('hello, world')"),
            H.unsafe("You can pass <i>raw</i> HTML with unsafe tag."),
        ),
    )

    snapshot.assert_match(prettify(dom), get_snapshot_name())



from __future__ import annotations

from typing import NewType, Optional

from typing_extensions import Unpack

from haitch._attrs import Attributes, GlobalAttrs
from haitch._elements._element import Element
from haitch._typing import Child

AsideElement = NewType("AsideElement", Element)
"""An `<aside>` element."""


def aside(
    *children: Child,
    extra_attrs: Optional[Attributes] = None,
    **attrs: Unpack[GlobalAttrs],
) -> AsideElement:
    """Represents a portion whose content is only indirectly related.

    Asides are frequently presented as sidebars or call-out boxes.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/aside>
    """
    el = Element("aside")(**attrs, **extra_attrs or {})(*children)
    return AsideElement(el)



from typing import Iterable

from haitch._elements._body import body
from haitch._elements._element import Element
from haitch._elements._head import head
from haitch._elements._html import html
from haitch._elements._script import ScriptElement
from haitch._elements._title import title
from haitch._typing import SupportsHtml
from haitch._void_elements._link import LinkElement
from haitch._void_elements._meta import meta


def html5(
    *,
    content: SupportsHtml,
    page_title: str = "",
    language_code: str = "",
    body_classes: str = "",
    links: Iterable[LinkElement] = (),
    scripts: Iterable[ScriptElement] = (),
) -> Element:
    """Base HTML5 document container.

    DEPRECATED: this component will not be shipped with the v1 release!
    """
    return html(lang=language_code)(
        head(
            meta(charset="utf-8"),
            meta(name="viewport", content="width=device-width, initial-scale=1"),
            meta(http_equiv="x-ua-compatible", content="ie=edge"),
            page_title and title(page_title),
            links,
            scripts,
        ),
        body(class_=body_classes)(content) if body_classes else body(content),
    )



from __future__ import annotations

from typing import NewType, Optional

from typing_extensions import Unpack

from haitch._attrs import Attributes, GlobalAttrs
from haitch._elements._element import Element
from haitch._typing import Child

SectionElement = NewType("SectionElement", Element)
"""A `<section>` element."""


def section(
    *children: Child,
    extra_attrs: Optional[Attributes] = None,
    **attrs: Unpack[GlobalAttrs],
) -> SectionElement:
    """Represents a generic standalone section of a document.

    Sections should always have a heading, with very few exceptions. It should
    only be used if there isn't a more specific element to represent it.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/section>
    """
    el = Element("section")(**attrs, **extra_attrs or {})(*children)
    return SectionElement(el)



from __future__ import annotations

from typing import NewType, Optional

from typing_extensions import Unpack

from haitch._attrs import Attributes, GlobalAttrs
from haitch._elements._element import Element
from haitch._typing import Child

UlElement = NewType("UlElement", Element)
"""A `<ul>` element."""


def ul(
    *children: Child,
    extra_attrs: Optional[Attributes] = None,
    **attrs: Unpack[GlobalAttrs],
) -> UlElement:
    """Represents an unordered list of items.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/ul>
    """
    el = Element("ul")(**attrs, **extra_attrs or {})(*children)
    return UlElement(el)



from __future__ import annotations

from typing import NewType, Optional

from typing_extensions import Unpack

from haitch._attrs import Attributes, GlobalAttrs
from haitch._elements._element import Element
from haitch._typing import Child

DivElement = NewType("DivElement", Element)
"""A `<div>` element."""


def div(
    *children: Child,
    extra_attrs: Optional[Attributes] = None,
    **attrs: Unpack[GlobalAttrs],
) -> DivElement:
    """Generic container for flow content.

    As a "pure" container, the `<div>` element does not inherently represent
    anything. Instead, it's used to group content so it can be easily styled
    using the `class` or `id` attributes, marking a section of a document as
    being written in a different language (using the `lang` attribute), and so
    on.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/div>
    """
    el = Element("div")(**attrs, **extra_attrs or {})(*children)
    return DivElement(el)



from __future__ import annotations

from typing import NewType, Optional

from typing_extensions import Unpack

from haitch._attrs import Attributes, GlobalAttrs
from haitch._elements._element import Element
from haitch._typing import Child

SummaryElement = NewType("SummaryElement", Element)
"""A `<summary>` element."""


def summary(
    *children: Child,
    extra_attrs: Optional[Attributes] = None,
    **attrs: Unpack[GlobalAttrs],
) -> SummaryElement:
    """Specifies a summary, caption, or legend for a `<details>` element.

    Clicking the `<summary>` element toggles the state of the parent `<details>`
    element open and closed.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/summary>
    """
    el = Element("summary")(**attrs, **extra_attrs or {})(*children)
    return SummaryElement(el)



from __future__ import annotations

from typing import NewType, Optional

from typing_extensions import NotRequired, Unpack

from haitch._attrs import Attributes, GlobalAttrs
from haitch._void_elements._void_element import VoidElement

ColElement = NewType("ColElement", VoidElement)
"""A `<col>` element."""


def col(
    *,
    extra_attrs: Optional[Attributes] = None,
    **attrs: Unpack[ColAttrs],
) -> ColElement:
    """Defines one or more columns in a column group.

    Note: The `<col>` element is only valid as a child of a `<colgroup>` element
    that has no span attribute defined.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/col>
    """
    el = VoidElement("col")(**attrs, **extra_attrs or {})
    return ColElement(el)


class ColAttrs(GlobalAttrs):
    """Attributes for the `<col>` element.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/col#attributes>
    """

    span: NotRequired[int]
    """Specifies the number of consecutive columns the <col> element spans.

    Warning: the value must be a positive integer greater than zero. If not
    present, its default value is 1.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/col#span>
    """



from __future__ import annotations

from typing import NewType, Optional

from typing_extensions import Unpack

from haitch._attrs import Attributes, GlobalAttrs
from haitch._void_elements._void_element import VoidElement

EmbedElement = NewType("EmbedElement", VoidElement)
"""An `<embed>` element."""


def embed(
    *,
    extra_attrs: Optional[Attributes] = None,
    **attrs: Unpack[EmbedAttrs],
) -> EmbedElement:
    """Embeds external content at the specified point in the document.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/embed>
    """
    el = VoidElement("embed")(**attrs, **extra_attrs or {})
    return EmbedElement(el)


class EmbedAttrs(GlobalAttrs):
    """Attributes for the `<embed>` element.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/embed#attributes>
    """

    height: int
    """The displayed height of the resource, in CSS pixels.

    Note: this must be an absolute value; percentages are not allowed.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/embed#height>
    """

    src: str
    """The URL of the resource being embedded.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/embed#src>
    """

    type_: str
    """The MIME type to use to select the plug-in to instantiate.

    Valid mime types: <https://www.iana.org/assignments/media-types/media-types.xhtml>

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/embed#type>
    """

    width: int
    """The displayed width of the resource, in CSS pixels.

    Note: this must be an absolute value; percentages are not allowed.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/embed#width>
    """



from __future__ import annotations

from typing import Literal, Mapping

from haitch._attrs import AttributeValue, serialize_attribute
from haitch._typing import Html


class VoidElement:
    """Lazily built HTML Void Element.

    <https://developer.mozilla.org/en-US/docs/Glossary/Void_element>
    """

    def __init__(self, tag: VoidElementTag) -> None:
        """Initialize element by providing a void tag name."""
        self._tag = tag
        self._attrs: Mapping[str, AttributeValue] = {}

    def __call__(self, **attrs: AttributeValue) -> VoidElement:
        """Add attributes to void element."""
        self._attrs = {**self._attrs, **attrs}
        return self

    def __str__(self) -> Html:
        """Renders the HTML Void Element as a string."""
        return Html(self._render())

    def _render(self) -> str:
        attrs_ = "".join(serialize_attribute(k, v) for k, v in self._attrs.items())
        return "<%(tag)s%(attrs)s/>" % {"tag": self._tag, "attrs": attrs_}


VoidElementTag = Literal[
    "area",
    "base",
    "br",
    "col",
    "embed",
    "hr",
    "img",
    "input",
    "line",
    "link",
    "meta",
    "source",
    "track",
    "wbr",
]



from haitch._elements._element import Element
from haitch._typing import Child


def fragment(*children: Child) -> Element:
    """Accepts only children as input and does not wrap its parent tag."""
    return Element("fragment")(*children)



from __future__ import annotations

from typing import NewType, Optional

from typing_extensions import Unpack

from haitch._attrs import Attributes, GlobalAttrs
from haitch._elements._element import Element
from haitch._typing import Child

HeaderElement = NewType("HeaderElement", Element)
"""A `<header>` element."""


def header(
    *children: Child,
    extra_attrs: Optional[Attributes] = None,
    **attrs: Unpack[GlobalAttrs],
) -> HeaderElement:
    """Represents introductory content.

    This is typically a group of introductory or nav aids. It may contain some
    heading elements but also a logo, a search form, an author name, and other
    elements.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/header>
    """
    el = Element("header")(**attrs, **extra_attrs or {})(*children)
    return HeaderElement(el)



from __future__ import annotations

from typing import NewType, Optional

from typing_extensions import NotRequired, Unpack

from haitch._attrs import Attributes, GlobalAttrs
from haitch._elements._element import Element
from haitch._typing import Child

LiElement = NewType("LiElement", Element)
"""A `<li>` element."""


def li(
    *children: Child,
    extra_attrs: Optional[Attributes] = None,
    **attrs: Unpack[LiAttrs],
) -> LiElement:
    """Represents a list item.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/li>
    """
    el = Element("li")(**attrs, **extra_attrs or {})(*children)
    return LiElement(el)


class LiAttrs(GlobalAttrs):
    """Attributes for the `<li>` element.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/li#attributes>
    """

    value: NotRequired[int]
    """Indicates the current ordinal value of the list item.

    The only allowed value for this attribute is a number, even if the list is
    displayed with Roman numerals or letters. List items that follow this one
    continue numbering from the value set. The value attribute has no meaning
    for unordered lists (`<ul>`) or for menus (`<menu>`).

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/li#value>
    """



from __future__ import annotations

from typing import NewType, Optional

from typing_extensions import Unpack

from haitch._attrs import Attributes, GlobalAttrs
from haitch._elements._element import Element
from haitch._typing import Child

NavElement = NewType("NavElement", Element)
"""A `<nav>` element."""


def nav(
    *children: Child,
    extra_attrs: Optional[Attributes] = None,
    **attrs: Unpack[GlobalAttrs],
) -> NavElement:
    """Represents a section whose purpose is to provide navigation links.

    Common examples of navigation sections are menus, tables of contents, and
    indexes. A document may have several `<nav>` elements.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/nav>
    """
    el = Element("nav")(**attrs, **extra_attrs or {})(*children)
    return NavElement(el)



from pytest_snapshot.plugin import Snapshot

import haitch as H
from tests.helpers import get_snapshot_name, prettify


def test_void_elements(snapshot: Snapshot) -> None:
    dom = H.fragment(
        H.map(
            H.area(
                shape="circle",
                coords="75,75,75",
                href="left.html",
                alt="Click to go left",
            ),
        ),
        H.base(href="https://www.example.com/"),
        H.embed(type_="video/mp4", src="flower.mp4", height=250, width=250),
        H.p("Fernstraßen", H.wbr(), "bau"),
    )

    snapshot.assert_match(prettify(dom), get_snapshot_name())



from pytest_snapshot.plugin import Snapshot

import haitch as H
from tests.helpers import get_snapshot_name, prettify


def test_table(snapshot: Snapshot) -> None:
    dom = H.table(
        H.colgroup(
            H.col(span=2),
        ),
        H.tr(
            H.th("Name"),
            H.th("Age"),
        ),
        H.tr(
            H.td("Maria Sanchez"),
            H.td("28"),
        ),
        H.tr(
            H.td("Michael Johnson"),
            H.td("34"),
        ),
    )

    snapshot.assert_match(prettify(dom), get_snapshot_name())



from __future__ import annotations

from typing import NewType, Optional

from typing_extensions import NotRequired, Unpack

from haitch._attrs import Attributes, GlobalAttrs
from haitch._elements._element import Element
from haitch._typing import Child

BodyElement = NewType("BodyElement", Element)
"""A `<body>` element."""


def body(
    *children: Child,
    extra_attrs: Optional[Attributes] = None,
    **attrs: Unpack[BodyAttrs],
) -> BodyElement:
    """Represents the content of an HTML document.

    Note: there can only be one `<body>` element in a document.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/body>
    """
    el = Element("body")(**attrs, **extra_attrs or {})(*children)
    return BodyElement(el)


class BodyAttrs(GlobalAttrs):
    """Attributes for the `<body>` element.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/body#attributes>
    """

    onafterprint: NotRequired[str]
    """Function to call after the user has printed the document.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/body#onafterprint>
    """

    onbeforeprint: NotRequired[str]
    """Function to call when the user requests printing of the document.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/body#onbeforeprint>
    """

    onbeforeunload: NotRequired[str]
    """Function to call when the document is about to be unloaded.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/body#onbeforeunload>
    """

    onblur: NotRequired[str]
    """Function to call when the document loses focus.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/body#onblur>
    """

    onerror: NotRequired[str]
    """Function to call when the document fails to load properly.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/body#onerror>
    """

    onfocus: NotRequired[str]
    """Function to call when the document receives focus.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/body#onfocus>
    """

    onhashchange: NotRequired[str]
    """Function to call when the fragment id part of the document's current address changed.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/body#onhashchange>
    """

    onlanguagechange: NotRequired[str]
    """Function to call when the preferred language changed.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/body#onlanguagechange>
    """

    onload: NotRequired[str]
    """Function to call when the document has finished loading.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/body#onload>
    """

    onmessage: NotRequired[str]
    """Function to call when the document has received a message.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/body#onmessage>
    """

    onoffline: NotRequired[str]
    """Function to call when network communication has failed.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/body#onoffline>
    """

    ononline: NotRequired[str]
    """Function to call when network communication has been restored.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/body#ononline>
    """

    onpopstate: NotRequired[str]
    """Function to call when the user has navigated session history.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/body#onpopstate>
    """

    onredo: NotRequired[str]
    """Function to call when user moves forward in undo transaction history.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/body#onredo>
    """

    onresize: NotRequired[str]
    """Function to call when the document has been resized.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/body#onresize>
    """

    onstorage: NotRequired[str]
    """Function to call when the storage area has changed.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/body#onstorage>
    """

    onundo: NotRequired[str]
    """Function to call when user moves backward in undo transaction history.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/body#onundo>
    """

    onunload: NotRequired[str]
    """Function to call when the document is going away.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/body#onunload>
    """



from __future__ import annotations

from typing import NewType, Optional

from typing_extensions import Unpack

from haitch._attrs import Attributes, GlobalAttrs
from haitch._elements._element import Element
from haitch._typing import Child

ArticleElement = NewType("ArticleElement", Element)
"""An `<article>` element."""


def article(
    *children: Child,
    extra_attrs: Optional[Attributes] = None,
    **attrs: Unpack[GlobalAttrs],
) -> ArticleElement:
    """Represents self-contained composition.

    This can occur in a document, page, application, or site.

    The `<article>` element is intended to be independently distributable or
    reusable (e.g., in syndication). Examples include: a forum post, a magazine
    or newspaper article, or a blog entry, a product card, a user-submitted
    comment, an interactive widget or gadget, or any other independent item of
    content.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/article>
    """
    el = Element("article")(**attrs, **extra_attrs or {})(*children)
    return ArticleElement(el)




from pytest_snapshot.plugin import Snapshot

import haitch as H
from tests.helpers import get_snapshot_name, prettify


def test_headings(snapshot: Snapshot) -> None:
    dom = H.fragment(
        H.h1("Heading 1"),
        H.h2("Heading 2"),
        H.h3("Heading 3"),
        H.h4("Heading 4"),
        H.h5("Heading 5"),
        H.h6("Heading 6"),
    )

    snapshot.assert_match(prettify(dom), get_snapshot_name())



from __future__ import annotations

from typing import NewType, Optional

from typing_extensions import Unpack

from haitch._attrs import Attributes, GlobalAttrs
from haitch._elements._element import Element
from haitch._typing import Child

TrElement = NewType("TrElement", Element)
"""A `<tr>` element."""


def tr(
    *children: Child,
    extra_attrs: Optional[Attributes] = None,
    **attrs: Unpack[GlobalAttrs],
) -> TrElement:
    """Defines a row of cells in a table.

    The row's cells can then be established using a mix of `<td>` (data cell)
    and `<th>` (header cell) elements.

    <https://developer.mozilla.org/en-US/docs/Web/HTML/Element/tr>
    """
    el = Element("tr")(**attrs, **extra_attrs or {})(*children)
    return TrElement(el)



import inspect

from bs4 import BeautifulSoup as bs
from bs4.formatter import HTMLFormatter

from haitch import SupportsHtml


def prettify(dom: SupportsHtml) -> str | bytes:
    """Pretty format rendered HTML to make it easier to spot diffs."""
    formatter = HTMLFormatter(indent=2, empty_attributes_are_booleans=True)
    soup = bs(str(dom), features="html.parser")
    return soup.prettify(formatter=formatter)


def get_snapshot_name() -> str:
    """Derive the snapshot name based on test function name."""
    function_name = inspect.currentframe().f_back.f_code.co_name  # type: ignore
    return f"{function_name}.html"
